@@ -23,11 +23,10 @@ with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include <string.h>
 #include <glib.h>
 #include <glib-object.h>
+#include <glib/gi18n.h>
 #include <gtk/gtk.h>
 #include <libdbusmenu-gtk/menu.h>
 #include <libdbusmenu-gtk/menuitem.h>
-#include <dbus/dbus-glib.h>
-#include <dbus/dbus-glib-bindings.h>
 
 #include <libindicator/indicator.h>
 #include <libindicator/indicator-object.h>
@@ -35,7 +34,7 @@ with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include <libindicator/indicator-service-manager.h>
 
 #include "dbus-data.h"
-#include "messages-service-client.h"
+#include "gen-messages-service.xml.h"
 
 #define INDICATOR_MESSAGES_TYPE            (indicator_messages_get_type ())
 #define INDICATOR_MESSAGES(obj)            (G_TYPE_CHECK_INSTANCE_CAST ((obj), INDICATOR_MESSAGES_TYPE, IndicatorMessages))
@@ -54,6 +53,7 @@ typedef struct _IndicatorMessagesClass IndicatorMessagesClass;
 
 struct _IndicatorMessagesClass {
 	IndicatorObjectClass parent_class;
+	void    (*update_a11y_desc) (IndicatorServiceManager * service, gpointer * user_data);
 };
 
 struct _IndicatorMessages {
@@ -69,8 +69,12 @@ INDICATOR_SET_TYPE(INDICATOR_MESSAGES_TYPE)
 
 /* Globals */
 static GtkWidget * main_image = NULL;
-static DBusGProxy * icon_proxy = NULL;
+static GDBusProxy * icon_proxy = NULL;
 static GtkSizeGroup * indicator_right_group = NULL;
+static GDBusNodeInfo *            bus_node_info = NULL;
+static GDBusInterfaceInfo *       bus_interface_info = NULL;
+static const gchar *              accessible_desc = NULL;
+static IndicatorObject *          indicator = NULL;
 
 /* Prototypes */
 static void indicator_messages_class_init (IndicatorMessagesClass *klass);
@@ -79,12 +83,34 @@ static void indicator_messages_dispose    (GObject *object);
 static void indicator_messages_finalize   (GObject *object);
 static GtkImage * get_icon                (IndicatorObject * io);
 static GtkMenu * get_menu                 (IndicatorObject * io);
+static const gchar * get_accessible_desc      (IndicatorObject * io);
 static void connection_change             (IndicatorServiceManager * sm,
                                            gboolean connected,
                                            gpointer user_data);
 
 G_DEFINE_TYPE (IndicatorMessages, indicator_messages, INDICATOR_OBJECT_TYPE);
 
+static void
+update_a11y_desc (void)
+{
+	g_return_if_fail(IS_INDICATOR_MESSAGES(indicator));
+
+	GList *entries = indicator_object_get_entries(indicator);
+	IndicatorObjectEntry * entry = (IndicatorObjectEntry *)entries->data;
+
+	entry->accessible_desc = get_accessible_desc(indicator);
+
+	g_signal_emit(G_OBJECT(indicator),
+	              INDICATOR_OBJECT_SIGNAL_ACCESSIBLE_DESC_UPDATE_ID,
+	              0,
+	              entry,
+	              TRUE);
+
+	g_list_free(entries);
+
+	return;
+}
+
 /* Initialize the one-timers */
 static void
 indicator_messages_class_init (IndicatorMessagesClass *klass)
@@ -98,6 +124,25 @@ indicator_messages_class_init (IndicatorMessagesClass *klass)
 
 	io_class->get_image = get_icon;
 	io_class->get_menu = get_menu;
+	io_class->get_accessible_desc = get_accessible_desc;
+
+	if (bus_node_info == NULL) {
+		GError * error = NULL;
+
+		bus_node_info = g_dbus_node_info_new_for_xml(_messages_service, &error);
+		if (error != NULL) {
+			g_error("Unable to parse Messaging Menu Interface description: %s", error->message);
+			g_error_free(error);
+		}
+	}
+
+	if (bus_interface_info == NULL) {
+		bus_interface_info = g_dbus_node_info_lookup_interface(bus_node_info, INDICATOR_MESSAGES_DBUS_SERVICE_INTERFACE);
+
+		if (bus_interface_info == NULL) {
+			g_error("Unable to find interface '" INDICATOR_MESSAGES_DBUS_SERVICE_INTERFACE "'");
+		}
+	}
 
 	return;
 }
@@ -113,6 +158,8 @@ indicator_messages_init (IndicatorMessages *self)
 	self->service = indicator_service_manager_new_version(INDICATOR_MESSAGES_DBUS_NAME, 1);
 	g_signal_connect(self->service, INDICATOR_SERVICE_MANAGER_SIGNAL_CONNECTION_CHANGE, G_CALLBACK(connection_change), self);
 
+	indicator = INDICATOR_OBJECT(self);
+
 	return;
 }
 
@@ -145,54 +192,85 @@ indicator_messages_finalize (GObject *object)
 
 /* Functions */
 
-/* Called everytime the attention changes in the service. */
+/* Signal off of the proxy */
 static void
-attention_changed_cb (DBusGProxy * proxy, gboolean dot, gpointer userdata)
+proxy_signal (GDBusProxy * proxy, const gchar * sender, const gchar * signal, GVariant * params, gpointer user_data)
 {
-	if (dot) {
-		indicator_image_helper_update(GTK_IMAGE(main_image), "indicator-messages-new");
-	} else {
-		indicator_image_helper_update(GTK_IMAGE(main_image), "indicator-messages");
-	}
-	return;
-}
+	gboolean prop = g_variant_get_boolean(g_variant_get_child_value(params, 0));
 
-/* Change the icon to whether it should be visible or not */
-static void
-icon_changed_cb (DBusGProxy * proxy, gboolean hidden, gpointer userdata)
-{
-	if (hidden) {
-		gtk_widget_hide(main_image);
+	if (g_strcmp0("AttentionChanged", signal) == 0) {
+		if (prop) {
+			indicator_image_helper_update(GTK_IMAGE(main_image), "indicator-messages-new");
+			accessible_desc = _("New Messages");
+		} else {
+			indicator_image_helper_update(GTK_IMAGE(main_image), "indicator-messages");
+			accessible_desc = _("Messages");
+		}
+	} else if (g_strcmp0("IconChanged", signal) == 0) {
+		if (prop) {
+			gtk_widget_hide(main_image);
+		} else {
+			gtk_widget_show(main_image);
+		}
 	} else {
-		gtk_widget_show(main_image);
+		g_warning("Unknown signal %s", signal);
 	}
+
+	update_a11y_desc();
+
 	return;
 }
 
 /* Callback from getting the attention status from the service. */
 static void
-attention_cb (DBusGProxy * proxy, gboolean dot, GError * error, gpointer userdata)
+attention_cb (GObject * object, GAsyncResult * ares, gpointer user_data)
 {
+	GError * error = NULL;
+	GVariant * res = g_dbus_proxy_call_finish(G_DBUS_PROXY(object), ares, &error);
+
 	if (error != NULL) {
 		g_warning("Unable to get attention status: %s", error->message);
 		g_error_free(error);
 		return;
 	}
 
-	return attention_changed_cb(proxy, dot, userdata);
+	gboolean prop = g_variant_get_boolean(g_variant_get_child_value(res, 0));
+
+	if (prop) {
+		indicator_image_helper_update(GTK_IMAGE(main_image), "indicator-messages-new");
+		accessible_desc = _("New Messages");
+	} else {
+		indicator_image_helper_update(GTK_IMAGE(main_image), "indicator-messages");
+		accessible_desc = _("Messages");
+	}
+
+	update_a11y_desc();
+
+	return;
 }
 
 /* Change from getting the icon visibility from the service */
 static void
-icon_cb (DBusGProxy * proxy, gboolean hidden, GError * error, gpointer userdata)
+icon_cb (GObject * object, GAsyncResult * ares, gpointer user_data)
 {
+	GError * error = NULL;
+	GVariant * res = g_dbus_proxy_call_finish(G_DBUS_PROXY(object), ares, &error);
+
 	if (error != NULL) {
 		g_warning("Unable to get icon visibility: %s", error->message);
 		g_error_free(error);
 		return;
 	}
 
-	return icon_changed_cb(proxy, hidden, userdata);
+	gboolean prop = g_variant_get_boolean(g_variant_get_child_value(res, 0));
+	
+	if (prop) {
+		gtk_widget_hide(main_image);
+	} else {
+		gtk_widget_show(main_image);
+	}
+
+	return;
 }
 
 static guint connection_drop_timeout = 0;
@@ -209,6 +287,43 @@ connection_drop_cb (gpointer user_data)
 	return FALSE;
 }
 
+/* Proxy is setup now.. whoo! */
+static void
+proxy_ready_cb (GObject * obj, GAsyncResult * res, gpointer user_data)
+{
+	GError * error = NULL;
+	GDBusProxy * proxy = g_dbus_proxy_new_for_bus_finish(res, &error);
+
+	if (error != NULL) {
+		g_warning("Unable to get proxy of service: %s", error->message);
+		g_error_free(error);
+		return;
+	}
+
+	icon_proxy = proxy;
+
+	g_signal_connect(G_OBJECT(proxy), "g-signal", G_CALLBACK(proxy_signal), user_data);
+
+	g_dbus_proxy_call(icon_proxy,
+	                  "AttentionRequested",
+	                  NULL, /* params */
+	                  G_DBUS_CALL_FLAGS_NONE,
+	                  -1, /* timeout */
+	                  NULL, /* cancel */
+	                  attention_cb,
+	                  user_data);
+	g_dbus_proxy_call(icon_proxy,
+	                  "IconShown",
+	                  NULL, /* params */
+	                  G_DBUS_CALL_FLAGS_NONE,
+	                  -1, /* timeout */
+	                  NULL, /* cancel */
+	                  icon_cb,
+	                  user_data);
+
+	return;
+}
+
 /* Sets up all the icon information in the proxy. */
 static void 
 connection_change (IndicatorServiceManager * sm, gboolean connected, gpointer user_data)
@@ -225,51 +340,46 @@ connection_change (IndicatorServiceManager * sm, gboolean connected, gpointer us
 		return;
 	}
 
-	DBusGConnection * connection = dbus_g_bus_get(DBUS_BUS_SESSION, NULL);
-	if (connection == NULL) {
-		g_warning("Unable to get session bus");
-		return;
-	}
-
 	if (icon_proxy == NULL) {
-		icon_proxy = dbus_g_proxy_new_for_name(connection,
-		                                       INDICATOR_MESSAGES_DBUS_NAME,
-		                                       INDICATOR_MESSAGES_DBUS_SERVICE_OBJECT,
-		                                       INDICATOR_MESSAGES_DBUS_SERVICE_INTERFACE);
-		if (icon_proxy == NULL) {
-			g_warning("Unable to get messages service interface.");
-			return;
-		}
-		
-		dbus_g_proxy_add_signal(icon_proxy, "AttentionChanged", G_TYPE_BOOLEAN, G_TYPE_INVALID);
-		dbus_g_proxy_connect_signal(icon_proxy,
-		                            "AttentionChanged",
-		                            G_CALLBACK(attention_changed_cb),
-		                            NULL,
-		                            NULL);
-
-		dbus_g_proxy_add_signal(icon_proxy, "IconChanged", G_TYPE_BOOLEAN, G_TYPE_INVALID);
-		dbus_g_proxy_connect_signal(icon_proxy,
-		                            "IconChanged",
-		                            G_CALLBACK(icon_changed_cb),
-		                            NULL,
-		                            NULL);
+		g_dbus_proxy_new_for_bus(G_BUS_TYPE_SESSION,
+		                         G_DBUS_PROXY_FLAGS_NONE,
+		                         bus_interface_info,
+		                         INDICATOR_MESSAGES_DBUS_NAME,
+		                         INDICATOR_MESSAGES_DBUS_SERVICE_OBJECT,
+		                         INDICATOR_MESSAGES_DBUS_SERVICE_INTERFACE,
+		                         NULL, /* cancel */
+		                         proxy_ready_cb,
+		                         sm);
+	} else {
+		g_dbus_proxy_call(icon_proxy,
+		                  "AttentionRequested",
+		                  NULL, /* params */
+		                  G_DBUS_CALL_FLAGS_NONE,
+		                  -1, /* timeout */
+		                  NULL, /* cancel */
+		                  attention_cb,
+		                  sm);
+		g_dbus_proxy_call(icon_proxy,
+		                  "IconShown",
+		                  NULL, /* params */
+		                  G_DBUS_CALL_FLAGS_NONE,
+		                  -1, /* timeout */
+		                  NULL, /* cancel */
+		                  icon_cb,
+		                  sm);
 	}
 
-	org_ayatana_indicator_messages_service_attention_requested_async(icon_proxy, attention_cb, NULL);
-	org_ayatana_indicator_messages_service_icon_shown_async(icon_proxy, icon_cb, NULL);
-
 	return;
 }
 
 /* Sets the icon when it changes. */
 static void
-application_icon_change_cb (DbusmenuMenuitem * mi, gchar * prop, GValue * value, gpointer user_data)
+application_icon_change_cb (DbusmenuMenuitem * mi, gchar * prop, GVariant * value, gpointer user_data)
 {
 	if (!g_strcmp0(prop, APPLICATION_MENUITEM_PROP_ICON)) {
 		/* Set the main icon */
 		if (GTK_IS_IMAGE(user_data)) {
-			gtk_image_set_from_icon_name(GTK_IMAGE(user_data), g_value_get_string(value), GTK_ICON_SIZE_MENU);
+			gtk_image_set_from_icon_name(GTK_IMAGE(user_data), g_variant_get_string(value, NULL), GTK_ICON_SIZE_MENU);
 		}
 	}
 
@@ -278,12 +388,12 @@ application_icon_change_cb (DbusmenuMenuitem * mi, gchar * prop, GValue * value,
 
 /* Sets the label when it changes. */
 static void
-application_prop_change_cb (DbusmenuMenuitem * mi, gchar * prop, GValue * value, gpointer user_data)
+application_prop_change_cb (DbusmenuMenuitem * mi, gchar * prop, GVariant * value, gpointer user_data)
 {
 	if (!g_strcmp0(prop, APPLICATION_MENUITEM_PROP_NAME)) {
 		/* Set the main label */
 		if (GTK_IS_LABEL(user_data)) {
-			gtk_label_set_text(GTK_LABEL(user_data), g_value_get_string(value));
+			gtk_label_set_text(GTK_LABEL(user_data), g_variant_get_string(value, NULL));
 		}
 	}
 
@@ -299,6 +409,7 @@ application_prop_change_cb (DbusmenuMenuitem * mi, gchar * prop, GValue * value,
 static gboolean
 application_triangle_draw_cb (GtkWidget *widget, GdkEventExpose *event, gpointer data)
 {
+	GtkAllocation allocation;
 	GtkStyle *style;
 	cairo_t *cr;
 	int x, y, arrow_width, arrow_height;
@@ -316,11 +427,12 @@ application_triangle_draw_cb (GtkWidget *widget, GdkEventExpose *event, gpointer
 	/* set arrow position / dimensions */
 	arrow_width = 5; /* the pixel-based reference triangle is 5x9 */
 	arrow_height = 9;
-	x = widget->allocation.x;
-	y = widget->allocation.y + widget->allocation.height/2.0 - (double)arrow_height/2.0;
+	gtk_widget_get_allocation (widget, &allocation);
+	x = allocation.x;
+	y = allocation.y + allocation.height/2.0 - (double)arrow_height/2.0;
 
 	/* initialize cairo drawing area */
-	cr = (cairo_t*) gdk_cairo_create (widget->window);
+	cr = (cairo_t*) gdk_cairo_create (gtk_widget_get_window (widget));
 
 	/* set line width */	
 	cairo_set_line_width (cr, 1.0);
@@ -359,6 +471,7 @@ custom_cairo_rounded_rectangle (cairo_t *cr,
 static gboolean
 numbers_draw_cb (GtkWidget *widget, GdkEventExpose *event, gpointer data)
 {
+	GtkAllocation allocation;
 	GtkStyle *style;
 	cairo_t *cr;
 	double x, y, w, h;
@@ -371,10 +484,11 @@ numbers_draw_cb (GtkWidget *widget, GdkEventExpose *event, gpointer data)
 	style = gtk_widget_get_style (widget);
 
 	/* set arrow position / dimensions */
-	w = widget->allocation.width;
-	h = widget->allocation.height;
-	x = widget->allocation.x;
-	y = widget->allocation.y;
+	gtk_widget_get_allocation (widget, &allocation);
+	w = allocation.width;
+	h = allocation.height;
+	x = allocation.x;
+	y = allocation.y;
 
 	layout = gtk_label_get_layout (GTK_LABEL(widget));
 
@@ -385,7 +499,7 @@ numbers_draw_cb (GtkWidget *widget, GdkEventExpose *event, gpointer data)
 	font_size = pango_font_description_get_size (font_description); */
 
 	/* initialize cairo drawing area */
-	cr = (cairo_t*) gdk_cairo_create (widget->window);
+	cr = (cairo_t*) gdk_cairo_create (gtk_widget_get_window (widget));
 
 	/* set line width */	
 	cairo_set_line_width (cr, 1.0);
@@ -412,7 +526,7 @@ numbers_draw_cb (GtkWidget *widget, GdkEventExpose *event, gpointer data)
 /* Builds a menu item representing a running application in the
    messaging menu */
 static gboolean
-new_application_item (DbusmenuMenuitem * newitem, DbusmenuMenuitem * parent, DbusmenuClient * client)
+new_application_item (DbusmenuMenuitem * newitem, DbusmenuMenuitem * parent, DbusmenuClient * client, gpointer user_data)
 {
 	g_debug ("%s (\"%s\")", __func__, dbusmenu_menuitem_property_get(newitem, APPLICATION_MENUITEM_PROP_NAME));
 
@@ -468,14 +582,14 @@ struct _indicator_item_t {
 /* Whenever we have a property change on a DbusmenuMenuitem
    we need to be responsive to that. */
 static void
-indicator_prop_change_cb (DbusmenuMenuitem * mi, gchar * prop, GValue * value, indicator_item_t * mi_data)
+indicator_prop_change_cb (DbusmenuMenuitem * mi, gchar * prop, GVariant * value, indicator_item_t * mi_data)
 {
 	if (!g_strcmp0(prop, INDICATOR_MENUITEM_PROP_LABEL)) {
 		/* Set the main label */
-		gtk_label_set_text(GTK_LABEL(mi_data->label), g_value_get_string(value));
+		gtk_label_set_text(GTK_LABEL(mi_data->label), g_variant_get_string(value, NULL));
 	} else if (!g_strcmp0(prop, INDICATOR_MENUITEM_PROP_RIGHT)) {
 		/* Set the right label */
-		gtk_label_set_text(GTK_LABEL(mi_data->right), g_value_get_string(value));
+		gtk_label_set_text(GTK_LABEL(mi_data->right), g_variant_get_string(value, NULL));
 	} else if (!g_strcmp0(prop, INDICATOR_MENUITEM_PROP_ICON)) {
 		/* We don't use the value here, which is probably less efficient, 
 		   but it's easier to use the easy function.  And since th value
@@ -520,7 +634,7 @@ indicator_prop_change_cb (DbusmenuMenuitem * mi, gchar * prop, GValue * value, i
    shifting over and putting the icon in with some right
    side text that'll be determined by the service.  */
 static gboolean
-new_indicator_item (DbusmenuMenuitem * newitem, DbusmenuMenuitem * parent, DbusmenuClient * client)
+new_indicator_item (DbusmenuMenuitem * newitem, DbusmenuMenuitem * parent, DbusmenuClient * client, gpointer user_data)
 {
 	g_return_val_if_fail(DBUSMENU_IS_MENUITEM(newitem), FALSE);
 	g_return_val_if_fail(DBUSMENU_IS_GTKCLIENT(client), FALSE);
@@ -628,3 +742,10 @@ get_menu (IndicatorObject * io)
 
 	return GTK_MENU(menu);
 }
+
+/* Returns the accessible description of the indicator */
+static const gchar *
+get_accessible_desc (IndicatorObject * io)
+{
+	return accessible_desc;
+}
